#!/usr/bin/env python
# Software License Agreement (BSD License)
#
# Copyright (c) 2008, Willow Garage, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above
#    copyright notice, this list of conditions and the following
#    disclaimer in the documentation and/or other materials provided
#    with the distribution.
#  * Neither the name of Willow Garage, Inc. nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Revision $Id: __init__.py 3803 2009-02-11 02:04:39Z rob_wheeler $

import sys
import os
import tempfile
import traceback

from rdcore import *

def main():
    filters = sys.argv[1:]
    
    # Load the ROS environment
    ctx = RosdocContext('doc', filters)
    try:
        ctx.init()
    except Exception, e:
        traceback.print_exc()
        sys.exit(1)

    try:
        if 1:
            # Generate Doxygen 
            import doxygenator
            success = doxygenator.generate_doxygen(ctx)
        else:
            #success = []
            success = ctx.packages.keys()

        if 1:
            # Generate Documentation Index
            import docindex 
            doc_index = os.path.join(ctx.docdir, 'index.html')
            docindex.generate_doc_index(ctx, success, doc_index)

        if 1:
            # Generate License Index
            import licenseindex
            license_index = os.path.join(ctx.docdir, 'licenses.html')
            licenseindex.generate_license_index(ctx, license_index)

        if 1:
            # support files
            import shutil
            styles_in = os.path.join(ctx.template_dir, 'styles.css')
            styles_css = os.path.join(ctx.docdir, 'styles.css')
            shutil.copyfile(styles_in, styles_css)
            
            # have to copy doxygen.css for external packages that we fake-doxygenate
            dstyles_in = os.path.join(ctx.template_dir, 'doxygen.css')
            dstyles_css = os.path.join(ctx.docdir, 'doxygen.css')
            shutil.copyfile(dstyles_in, dstyles_css)
    except:
        traceback.print_exc()
